@@ -2,7 +2,7 @@
 set_project("xmake")
 
 -- version
-set_version("2.9.2", {build = "%Y%m%d"})
+set_version("2.9.5", {build = "%Y%m%d"})
 
 -- set xmake min version
 set_xmakever("2.8.5")
@@ -48,6 +48,19 @@ if is_mode("coverage") then
     add_ldflags("-coverage", "-fprofile-arcs", "-ftest-coverage")
 end
 
+-- set cosmocc toolchain, e.g. xmake f -p linux --cosmocc=y
+if has_config("cosmocc") then
+    add_requires("cosmocc")
+    set_toolchains("@cosmocc")
+    set_policy("build.ccache", false)
+end
+
+-- use cosmocc toolchain
+option("cosmocc", {default = false, description = "Use cosmocc toolchain to build once and run anywhere."})
+
+-- embed all script files
+option("embed", {default = false, description = "Embed all script files."})
+
 -- the runtime option
 option("runtime")
     set_default("lua")
@@ -65,17 +78,36 @@ option_end()
 option("readline")
     set_description("Enable or disable readline library")
     add_links("readline")
-    add_cincludes("readline/readline.h")
+    add_cincludes("stdio.h", "readline/readline.h")
     add_cfuncs("readline")
     add_defines("XM_CONFIG_API_HAVE_READLINE")
+    add_deps("cosmocc")
+    after_check(function (option)
+        if option:dep("cosmocc"):enabled() then
+            option:enable(false)
+        end
+    end)
 option_end()
 
 -- the curses option
 option("curses")
     set_description("Enable or disable curses library")
-    add_links("curses")
-    add_cincludes("curses.h")
     add_defines("XM_CONFIG_API_HAVE_CURSES")
+    add_deps("cosmocc")
+    before_check(function (option)
+        if is_plat("mingw") then
+            option:add("cincludes", "ncursesw/curses.h")
+            option:add("links", "ncursesw")
+        else
+            option:add("cincludes", "curses.h")
+            option:add("links", "curses")
+        end
+    end)
+    after_check(function (option)
+        if option:dep("cosmocc"):enabled() then
+            option:enable(false)
+        end
+    end)
 option_end()
 
 -- the pdcurses option
